/************************************************************************
 *   IRC - Internet Relay Chat, m_core/m_burstdone.c
 *   This file is copyright (C) 2001 Andrew Suffield
 *                                    <asuffield@users.sourceforge.net>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 */

#include "ircd.h"
#include "common.h"
#include "numeric.h"
#include "irc_string.h"
#include "send.h"
#include "umodes.h"
#include "msg.h"
#include "handlers.h"
#include "channel.h"
#include "client.h"
#include "struct.h"
#include "burst.h"
#include "s_log.h"

#include <string.h>
#include <sys/time.h>

int m_burstdone(struct Client *, struct Client *, int, char **);

struct Message burstdone_msgtab = {
  "BURSTDONE", 0, 0, 0, MFLG_SLOW, 0,
  {m_unregistered, m_ignore, m_burstdone},
  NULL, NULL
};

int
m_burstdone(struct Client *cptr, struct Client *sptr, int parc, char *parv[])
{
  struct Channel *chptr;
  struct Client *acptr;

  if (local_burst)
    {
      /* OK, two possibilities. First case, we're in mid-burst, I just got this
       * In this case, my peer has finished bursting, and wants me to begin
       */
      if (burst_in_progress && burst_in_progress->serv->clacked)
	{
	  /* Time for me to burst */

	  burst_to(cptr);

	  sendto_ops_flag_butone(cptr, UMODE_SEEROUTING,
				 "Completed bursting with %s (%.1lu sec)",
				 sptr->name, CurrentTime - sptr->firsttime);
	  log(L_INFO, "Completed bursting with %s (%.1lu sec)",
	      sptr->name, CurrentTime - sptr->firsttime);
	}
      else
	{
	  /* Second case, we finished bursting, this is from my peer letting me know
	   * I can go do a final check and cleanup
	   */
	  sendto_ops_flag_butone(cptr, UMODE_SEEROUTING,
				 "%s acknowledged end of %s burst (%.1lu sec)",
				 sptr->name, me.name,
				 CurrentTime - sptr->firsttime);
	  log(L_INFO, "%s acknowledged end of %s burst (%.1lu sec)",
	      sptr->name, me.name,
	      CurrentTime - sptr->firsttime);
	}
      /* Do this stuff in both cases */
      /* Note that this will be called *ONCE* per burst, not once each way */
      gettimeofday(&cptr->ping_send_time, NULL);
      sendto_one(cptr, "PING :%s", me.name);

#ifdef ZIP_LINKS
      /*
       * some stats about the connect burst,
       * they are slightly incorrect because of cptr->zip->outbuf.
       */
      if ((cptr->flags2 & FLAGS2_ZIP) && cptr->zip->out->total_in)
	sendto_ops_flag(UMODE_SERVNOTICE,
			"Connect burst to %s: %lu, compressed: %lu (%3.1f%%)",
			get_client_name(cptr, MASK_IP),
			cptr->zip->out->total_in, cptr->zip->out->total_out,
			(100.0 * (float) cptr->zip->out->total_out) /
			(float) cptr->zip->out->total_in);
#endif
    }
  else
    {
      if (parc > 1)
	{
	  sendto_local_ops_flag_butflag(UMODE_SERVNOTICE, UMODE_SEEROUTING,
					"Burst complete, %s has linked",
					parv[1]);
	  log(L_INFO, "Burst complete, %s has linked",
	      parv[1]);
	}
      else
	{
	  sendto_ops_flag(UMODE_DEBUG, "WTF: BURSTDONE from %s", sptr->name);
	  log(L_WARN, "WTF: BURSTDONE from %s", sptr->name);
	}
    }

  /* This stuff happens in the cases where I've been told
   * that the burst is over and I can clean up as well
   */

  burst_in_progress = NULL;
  local_burst = 0;
  waiting_for_clacks = 0;
  already_clacked = 0;

  /* Let the servers behind me know that I'm done, so they
   * are allowed to accept new servers again
   */
  sendto_serv_butone(cptr, ":%s BURSTDONE %s", me.name, cptr->name);

  /* Search for any empty channels and clear them */
  /* Also clear frozen status from all channels */
  for (chptr = channel; chptr; chptr = chptr->nextch)
    {
      chptr->frozen_by = NULL;
      if (chptr->users == 0)
	sub1_from_channel(chptr);
    }

  for (acptr = serv_cptr_list; acptr; acptr = acptr->next_server_client)
    acptr->serv->clacked = 0;
  return 0;
}
